# Today's data is taken from
# https://data.cityofchicago.org/Transportation/Taxi-Trips/wrvz-psew
# It is filtered to reflect only taxi trips from December, 2020.


# make sure that you set the working directory correctly:
# Session > Set Working Directory > To Source File Location
# then execute these lines
# this reads in the file 'Taxi_Trips_-_2020_12.tsv' from the current directory
# since it is a tab-separated file, we need to set the separator explicitly with the parameter sep
taxi_trips <- read.csv("Taxi_Trips_-_2020_12.tsv", sep = "\t")
# you can click on the cs entry in the Environment-tab on the right to check the data
# important: it has to have 149056 observations of 23 variables.
# the observations are the number of rows and the variables the number of columns in your dataset.

# here, we have a so called data frame
class(taxi_trips)
# it is R's abstraction of tables
nrow(taxi_trips)
ncol(taxi_trips)
# we can access cells of the table using square brackets. here, we take the second row and the third column.
# in R it is always row,column
taxi_trips[2,3]
# the great thing is how we can work with the data inside the table's columns!
colnames(taxi_trips)
# each of the column name can be used to access the corresponding field in the table
taxi_trips$Trip.Miles
# yields a list with 149056 entries each specifying the length of the trip in miles
summary(taxi_trips$Trip.Miles)

# Let's see if there is a relation between the length of the trip and the fare
taxi_trips$Trip.Total
# we can visualize both data series using a simple plot
plot(taxi_trips$Trip.Miles, taxi_trips$Trip.Total)
# there is something strange: a trip with several hundred miles cost nothing
taxi_trips[which.max(taxi_trips$Trip.Miles),]
# and trips with length 0 cost a lot.
taxi_trips[which.max(taxi_trips$Trip.Total),]
# let's filter out these strange artifacts. 
# we begin with filtering zero-length trips
# this checks for each entry the condition of being greater than zero
taxi_trips$Trip.Miles > 0
# now we keep all rows that fulfil this condition.
# make sure to set that comma inside the square brackets to have the condition be evaluated on all rows!
taxi_clean <- taxi_trips[taxi_trips$Trip.Miles > 0, ]
# now the same thing for zero-cost trips
taxi_clean <- taxi_clean[taxi_clean$Trip.Total > 0, ]
# we filtered out a few thousand entries
nrow(taxi_clean)

plot(taxi_clean$Trip.Miles, taxi_clean$Trip.Total)
# hm, still some artifacts: going 950 miles per minute ;-)
# here, we would need a more elaborate way of filtering the dataset...
taxi_clean[which.max(taxi_clean$Trip.Miles),]
# maybe it has something to do with payment type
# the table command is often helpful to get a sense of textual data!
table(taxi_clean$Payment.Type)
# Let's focus on trips payed by credit card
taxi_clean <- taxi_clean[taxi_clean$Payment.Type == "Credit Card",]
plot(taxi_clean$Trip.Miles, taxi_clean$Trip.Total)

# Let's inspect these unplausible trips by selecting those rows with either
# zero-length trips or totals of zero
curious <- taxi_trips[taxi_trips$Trip.Miles == 0 | taxi_trips$Trip.Total == 0, ]
table(curious$Company)
# throw away empty 'levels'
curious <- droplevels(curious)
# maybe it's a case of the companies reporting skewed data
table(curious$Company)

# so we try to filter out trips conducted by these companies.
# therefore we formulate that trips of interest are longer
# than zero miles, cost more than zero dollar, and the company
# is not one of those companies identified above.
taxi_plausible <- taxi_trips[taxi_trips$Trip.Miles > 0 & 
                             taxi_trips$Trip.Total > 0 & 
                             !(taxi_trips$Company %in% curious$Company), ]
plot(taxi_plausible$Trip.Miles, taxi_plausible$Trip.Total)
# ok, this looks better, although the data shrinked considerably
# anyhow, let's assume a linear relation between both parameters
# and use that assumption to conduct a linear regression
# here, the ~ sign separates the dependent variable and the
# independent variable. Trip.Miles is the independent variable.
trip_model <- lm(taxi_plausible$Trip.Total ~ taxi_plausible$Trip.Miles)
# we can inspect the model
trip_model$coefficients
# and visualize it:
plot(taxi_plausible$Trip.Miles, taxi_plausible$Trip.Total)
# this takes the intercept and slope and adds a red line to the plot
abline(trip_model, col="red")

